@@ -1,11 +1,17 @@
 # -*- coding: utf-8 -*-
 
+import os
 import json
 import pytest
 import sys
 import tuxsuite.cli
 import tuxsuite.build
 
+from unittest.mock import MagicMock
+
+
+from requests import HTTPError
+
 
 sample_token = "Q9qMlmkjkIuIGmEAw-Mf53i_qoJ8Z2eGYCmrNx16ZLLQGrXAHRiN2ce5DGlAebOmnJFp9Ggcq9l6quZdDTtrkw"
 sample_url = "https://foo.bar.tuxbuild.com/v1"
@@ -179,7 +185,7 @@ class TestBakeCli:
         assert exc.value.code == 0
         assert exc.type == SystemExit
         output, error = capsys.readouterr()
-        assert "usage: tuxsuite bake [-h] {submit,cancel} ..." in output
+        assert "usage: tuxsuite bake [-h] {get,list,submit,cancel} ..." in output
 
     def test_bake(
         self, mocker, monkeypatch, tuxsuite_config, tmp_path, sample_bitbake_template
@@ -258,6 +264,62 @@ class TestBakeCli:
             == "Error: Either local manifest or pinned manifest to be provided, not both\n"
         )
 
+    def test_bake_json_out(
+        self, monkeypatch, mocker, tmp_path, tuxsuite_config, sample_bitbake_template
+    ):
+        template = tmp_path / "yocto.json"
+        template.write_text(sample_bitbake_template)
+        build = mocker.patch("tuxsuite.Bitbake")
+        build.return_value.status = {"sample": "value"}
+        monkeypatch.setattr(
+            sys,
+            "argv",
+            [
+                "tuxsuite",
+                "bake",
+                "submit",
+                f"{template}",
+                "--json-out",
+                f"{tmp_path}/bake.json",
+            ],
+        )
+        tuxsuite.cli.main()
+        build.assert_called_once()
+        assert os.path.exists(f"{tmp_path}/bake.json")
+
+    def test_bake_json_out_fail(
+        self,
+        monkeypatch,
+        mocker,
+        tmp_path,
+        tuxsuite_config,
+        sample_bitbake_template,
+        response,
+    ):
+        template = tmp_path / "yocto.json"
+        template.write_text(sample_bitbake_template)
+        build = mocker.patch(
+            "tuxsuite.Bitbake",
+            side_effect=HTTPError(mocker.Mock(status=500), "Unknown Error"),
+        )
+        build.return_value.status = {"sample": "value"}
+        monkeypatch.setattr(
+            sys,
+            "argv",
+            [
+                "tuxsuite",
+                "bake",
+                "submit",
+                f"{template}",
+                "--json-out",
+                f"{tmp_path}/bake.json",
+            ],
+        )
+        with pytest.raises(HTTPError):
+            tuxsuite.cli.main()
+        build.assert_called_once()
+        assert not os.path.exists(f"{tmp_path}/bake.json")
+
 
 class TestBuildCli:
     def test_build_no_args(self, monkeypatch, tuxsuite_config, capsys):
@@ -394,6 +456,8 @@ class TestBuildCli:
             patch_series=None,
             is_public=True,
             callback=None,
+            callback_headers=None,
+            notify_emails=None,
         )
 
     def test_build_download(self, mocker, monkeypatch, tuxsuite_config):
@@ -440,12 +504,17 @@ class TestBuildCli:
         tuxsuite.cli.main()
         download.assert_called_with(mocker.ANY, str(tmp_path))
 
-    def test_build_show_logs(self, mocker, monkeypatch, tuxsuite_config):
+    def test_build_show_logs(self, mocker, monkeypatch, tuxsuite_config, capsys):
         Build = mocker.patch("tuxsuite.Build")
         build = Build.return_value
         build.status = {}
         build.status["download_url"] = "https://builds.com/21321312312/"
-        download_file = mocker.patch("tuxsuite.download.download_file")
+        mock_get = mocker.patch("tuxsuite.download.get")
+        mock_response = MagicMock()
+        mock_response.iter_content = MagicMock(
+            return_value=[b"data_chunk1", b"data_chunk2"]
+        )
+        mock_get.return_value = mock_response
         monkeypatch.setattr(
             sys,
             "argv",
@@ -462,9 +531,9 @@ class TestBuildCli:
         )
         tuxsuite.cli.main()
         build.build.assert_called_once()
-        download_file.assert_called_with(
-            "https://builds.com/21321312312/build.log", mocker.ANY, headers=mocker.ANY
-        )
+        _, error = capsys.readouterr()
+        assert "data_chunk1" in error
+        assert "data_chunk2" in error
 
     def test_build_download_show_logs(
         self, mocker, monkeypatch, tuxsuite_config, tmp_path, capsys
@@ -585,6 +654,8 @@ class TestBuildCli:
             patch_series=None,
             is_public=True,
             callback=None,
+            callback_headers=None,
+            notify_emails=None,
         )
 
     def test_build_no_make_targets(self, mocker, monkeypatch, tuxsuite_config):
@@ -623,6 +694,8 @@ class TestBuildCli:
             patch_series=None,
             is_public=True,
             callback=None,
+            callback_headers=None,
+            notify_emails=None,
         )
 
     def test_build_valid_make_vars(self, mocker, monkeypatch, tuxsuite_config):
@@ -701,6 +774,8 @@ class TestBuildCli:
             patch_series=None,
             is_public=True,
             callback=None,
+            callback_headers=None,
+            notify_emails=None,
         )
 
     def test_build_no_wait(self, mocker, monkeypatch, tuxsuite_config, tmp_path):
@@ -722,6 +797,55 @@ class TestBuildCli:
         tuxsuite.cli.main()
         build.assert_called_once()
 
+    def test_build_json_out(self, mocker, monkeypatch, tuxsuite_config, tmp_path):
+        build = mocker.patch("tuxsuite.Build")
+        build.return_value.status = sample_build_result
+        monkeypatch.setattr(
+            sys,
+            "argv",
+            [
+                "tuxsuite",
+                "build",
+                "--git-repo=https://git.example.com/linux.git",
+                "--git-sha=beefbee",
+                "--target-arch=arm64",
+                "--kconfig=defconfig",
+                "--toolchain=gcc-9",
+                "--json-out",
+                f"{tmp_path}/build.json",
+            ],
+        )
+        tuxsuite.cli.main()
+        build.assert_called_once()
+        assert os.path.exists(f"{tmp_path}/build.json")
+
+    def test_build_json_out_fail(
+        self, mocker, monkeypatch, tuxsuite_config, tmp_path, response
+    ):
+        build = mocker.patch(
+            "tuxsuite.Build",
+            side_effect=HTTPError(mocker.Mock(status=500), "Unknown Error"),
+        )
+        monkeypatch.setattr(
+            sys,
+            "argv",
+            [
+                "tuxsuite",
+                "build",
+                "--git-repo=https://git.example.com/linux.git",
+                "--git-sha=beefbee",
+                "--target-arch=arm64",
+                "--kconfig=defconfig",
+                "--toolchain=gcc-9",
+                "--json-out",
+                f"{tmp_path}/build.json",
+            ],
+        )
+        with pytest.raises(Exception):
+            tuxsuite.cli.main()
+        build.assert_called_once()
+        assert not os.path.exists(f"{tmp_path}/build.json")
+
 
 class TestWaitForObject:
     def test_wait_for_object_pass(self, mocker, build_state):
@@ -826,9 +950,57 @@ class TestPlanCli:
             git_sha=None,
             patch_series=sample_patch_tgz,
             no_cache=False,
+            parameters=[],
             manifest_file=None,
             pinned_manifest=None,
+            kas_override=None,
             is_public=True,
+            lab="https://lkft.validation.linaro.org",
+            lava_test_plans_project=None,
+            callback=None,
+            callback_headers=None,
+            plan_callback=None,
+            plan_callback_headers=None,
+            notify_emails=None,
+        )
+        monkeypatch.setattr(
+            sys,
+            "argv",
+            [
+                "tuxsuite",
+                "plan",
+                "--git-repo=https://git.example.com/linux.git",
+                "--git-ref=master",
+                "--download",
+                "--show-logs",
+                str(plan_config),
+                f"--patch-series={sample_patch_tgz}",
+                "--parameters",
+                "a=b",
+                "--parameters",
+                "c=d",
+            ],
+        )
+        tuxsuite.cli.main()
+        plan.assert_called_with(
+            mocker.ANY,
+            git_repo="https://git.example.com/linux.git",
+            git_ref="master",
+            git_sha=None,
+            patch_series=sample_patch_tgz,
+            no_cache=False,
+            parameters=[("a", "b"), ("c", "d")],
+            manifest_file=None,
+            pinned_manifest=None,
+            kas_override=None,
+            is_public=True,
+            lab="https://lkft.validation.linaro.org",
+            lava_test_plans_project=None,
+            callback=None,
+            callback_headers=None,
+            plan_callback=None,
+            plan_callback_headers=None,
+            notify_emails=None,
         )
 
     def test_plan_empty_config(
@@ -868,6 +1040,54 @@ class TestPlanCli:
         tuxsuite.cli.main()
         plan.assert_called_once()
 
+    def test_plan_cli_json_out(
+        self, mocker, monkeypatch, tuxsuite_config, plan_config, tmp_path
+    ):
+        plan = mocker.patch("tuxsuite.Plan")
+        plan.return_value.status = sample_plan_status
+        monkeypatch.setattr(
+            sys,
+            "argv",
+            [
+                "tuxsuite",
+                "plan",
+                "--git-repo=https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git",
+                "--git-ref=https://example.com/vmlinux",
+                f"{plan_config}",
+                "--json-out",
+                f"{tmp_path}/plan.json",
+            ],
+        )
+        tuxsuite.cli.main()
+        plan.assert_called_once()
+        assert os.path.exists(f"{tmp_path}/plan.json")
+
+    def test_plan_cli_json_out_fail(
+        self, mocker, monkeypatch, tuxsuite_config, plan_config, tmp_path, response
+    ):
+        plan = mocker.patch(
+            "tuxsuite.Plan",
+            side_effect=HTTPError(mocker.Mock(status=500), "Unknown Error"),
+        )
+        plan.return_value.status = sample_plan_status
+        monkeypatch.setattr(
+            sys,
+            "argv",
+            [
+                "tuxsuite",
+                "plan",
+                "--git-repo=https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git",
+                "--git-ref=https://example.com/vmlinux",
+                f"{plan_config}",
+                "--json-out",
+                f"{tmp_path}/plan.json",
+            ],
+        )
+        with pytest.raises(Exception):
+            tuxsuite.cli.main()
+        plan.assert_called_once()
+        assert not os.path.exists(f"{tmp_path}/plan.json")
+
     def test_bake_plan_cli(
         self, mocker, monkeypatch, bake_plan_extended_config, tuxsuite_config, capsys
     ):
@@ -885,6 +1105,78 @@ class TestPlanCli:
         tuxsuite.cli.main()
         plan.assert_called_once()
 
+    def test_plan_callback_options(
+        self,
+        mocker,
+        monkeypatch,
+        plan_config,
+        tuxsuite_config,
+        capsys,
+    ):
+        # Error when plan callback header is provided without a plan callback
+        plan = mocker.patch("tuxsuite.Plan")
+        monkeypatch.setattr(
+            sys,
+            "argv",
+            [
+                "tuxsuite",
+                "plan",
+                "submit",
+                "--git-repo=https://git.example.com/linux.git",
+                "--git-ref=master",
+                str(plan_config),
+                "--plan-callback-header",
+                "X-H-a: h1",
+            ],
+        )
+        with pytest.raises(SystemExit) as exc:
+            tuxsuite.cli.main()
+        assert exc.value.code == 1
+        assert exc.type == SystemExit
+        output, error = capsys.readouterr()
+        assert "Error: --plan-callback-header given without a --plan-callback" in error
+
+        # happy flow
+        plan = mocker.patch("tuxsuite.Plan")
+        monkeypatch.setattr(
+            sys,
+            "argv",
+            [
+                "tuxsuite",
+                "plan",
+                "submit",
+                "--git-repo=https://git.example.com/linux.git",
+                "--git-ref=master",
+                str(plan_config),
+                "--plan-callback-header",
+                "X-H-a: h1",
+                "--plan-callback-header",
+                "X-H-b: h2",
+                "--plan-callback=https://www.example.com/plan_callback",
+            ],
+        )
+        tuxsuite.cli.main()
+        plan.assert_called_with(
+            mocker.ANY,
+            git_repo="https://git.example.com/linux.git",
+            git_ref="master",
+            git_sha=None,
+            no_cache=False,
+            manifest_file=None,
+            patch_series=None,
+            pinned_manifest=None,
+            parameters=[],
+            kas_override=None,
+            is_public=True,
+            lab="https://lkft.validation.linaro.org",
+            lava_test_plans_project=None,
+            callback=None,
+            callback_headers=None,
+            plan_callback="https://www.example.com/plan_callback",
+            plan_callback_headers={"X-H-a": "h1", "X-H-b": "h2"},
+            notify_emails=None,
+        )
+
 
 class TestTestCli:
     def test_test_no_args(self, monkeypatch, tuxsuite_config, capsys):
@@ -918,6 +1210,11 @@ class TestTestCli:
                 "test",
                 "--device=qemu-arm64",
                 "--kernel=https://example.com/vmlinux",
+                "--overlay",
+                "https://example.com/overlay",
+                "--overlay",
+                "https://example.com/overlay",
+                "/tmp",
             ],
         )
         tuxsuite.cli.main()
@@ -935,15 +1232,28 @@ class TestTestCli:
             scp_fw=None,
             scp_romfw=None,
             fip=None,
+            job_definition=None,
             parameters={},
             tests=[],
+            overlays=[
+                ["https://example.com/overlay", "/"],
+                ["https://example.com/overlay", "/tmp"],
+            ],
             timeouts={},
             boot_args=None,
             wait_for=None,
             callback=None,
+            callback_headers=None,
             commands=[],
             qemu_image=None,
             tuxbuild=None,
+            lab="https://lkft.validation.linaro.org",
+            lava_test_plans_project=None,
+            shared=False,
+            host="x86_64",
+            is_public=True,
+            test_name=None,
+            notify_emails=None,
         )
 
         mocker.resetall()
@@ -974,15 +1284,25 @@ class TestTestCli:
             scp_fw=None,
             scp_romfw=None,
             fip=None,
+            job_definition=None,
             parameters={},
             tests=["ltp-smoke"],
+            overlays=[],
             timeouts={},
             boot_args=None,
             wait_for=None,
             callback=None,
+            callback_headers=None,
             commands=[],
             qemu_image=None,
             tuxbuild=None,
+            lab="https://lkft.validation.linaro.org",
+            lava_test_plans_project=None,
+            shared=False,
+            host="x86_64",
+            is_public=True,
+            test_name=None,
+            notify_emails=None,
         )
 
         mocker.resetall()
@@ -1012,15 +1332,25 @@ class TestTestCli:
             scp_fw=None,
             scp_romfw=None,
             fip=None,
+            job_definition=None,
             parameters={},
             tests=["ltp-smoke"],
+            overlays=[],
             timeouts={},
             boot_args=None,
             wait_for="mybuilduid",
             callback=None,
+            callback_headers=None,
             commands=[],
             qemu_image=None,
             tuxbuild=None,
+            lab="https://lkft.validation.linaro.org",
+            lava_test_plans_project=None,
+            shared=False,
+            host="x86_64",
+            is_public=True,
+            test_name=None,
+            notify_emails=None,
         )
 
         mocker.resetall()
@@ -1050,15 +1380,25 @@ class TestTestCli:
             scp_fw=None,
             scp_romfw=None,
             fip=None,
+            job_definition=None,
             parameters={},
             tests=[],
+            overlays=[],
             timeouts={"deploy": 1, "boot": 2},
             boot_args=None,
             wait_for=None,
             callback=None,
+            callback_headers=None,
             commands=[],
             qemu_image=None,
             tuxbuild=None,
+            lab="https://lkft.validation.linaro.org",
+            lava_test_plans_project=None,
+            shared=False,
+            host="x86_64",
+            is_public=True,
+            test_name=None,
+            notify_emails=None,
         )
 
         mocker.resetall()
@@ -1096,15 +1436,25 @@ class TestTestCli:
             scp_fw="scp_fw.bin",
             scp_romfw="scp_romfw.bin",
             fip="fip.bin",
+            job_definition=None,
             parameters={"USERDATA": "userdata.tar.xz", "TC_URL": "toolchain.tar.xz"},
             tests=["lldb"],
+            overlays=[],
             timeouts={},
             boot_args=None,
             wait_for=None,
             callback=None,
+            callback_headers=None,
             commands=[],
             qemu_image=None,
             tuxbuild=None,
+            lab="https://lkft.validation.linaro.org",
+            lava_test_plans_project=None,
+            shared=False,
+            host="x86_64",
+            is_public=True,
+            test_name=None,
+            notify_emails=None,
         )
 
     def test_test_cli_errors(self, mocker, monkeypatch):
@@ -1138,6 +1488,48 @@ class TestTestCli:
         tuxsuite.cli.main()
         test.assert_called_once()
 
+    def test_test_cli_json_out(self, mocker, monkeypatch, tuxsuite_config, tmp_path):
+        test = mocker.patch("tuxsuite.Test")
+        test.return_value.status = sample_test_result
+        monkeypatch.setattr(
+            sys,
+            "argv",
+            [
+                "tuxsuite",
+                "test",
+                "--device=qemu-arm64",
+                "--kernel=https://example.com/vmlinux",
+                "--json-out",
+                f"{tmp_path}/test.json",
+            ],
+        )
+        tuxsuite.cli.main()
+        test.assert_called_once()
+        assert os.path.exists(f"{tmp_path}/test.json")
+
+    def test_test_cli_json_out_fail(
+        self, mocker, monkeypatch, tuxsuite_config, tmp_path, response
+    ):
+        test = mocker.patch(
+            "tuxsuite.Test",
+            side_effect=HTTPError(mocker.Mock(status=500), "Unknown Error"),
+        )
+        monkeypatch.setattr(
+            sys,
+            "argv",
+            [
+                "tuxsuite",
+                "test",
+                "--device=qemu-arm64",
+                "--kernel=https://example.com/vmlinux",
+                "--no-wait",
+            ],
+        )
+        with pytest.raises(Exception):
+            tuxsuite.cli.main()
+        test.assert_called_once()
+        assert not os.path.exists(f"{tmp_path}/test.json")
+
 
 sample_build_result = {
     "project": "ci",
@@ -1274,7 +1666,7 @@ sample_plan_status = {
     },
 }
 
-# sample data in list fromat to be consumed by from json for result status
+# sample data in list format to be consumed by from json for result status
 
 sample_list_data = [sample_build_result, sample_oebuild_result]
 
@@ -1606,7 +1998,7 @@ class TestBuildPatchSeries:
                 "--patch-series=httpsucks.patch",
             ],
         )
-        with pytest.raises(FileNotFoundError):
+        with pytest.raises(Exception):
             tuxsuite.cli.main()
 
         mocker.resetall()
@@ -1625,7 +2017,7 @@ class TestBuildPatchSeries:
                 "--patch-series=/tmp/invalidpatch",
             ],
         )
-        with pytest.raises(FileNotFoundError):
+        with pytest.raises(Exception):
             tuxsuite.cli.main()
 
     def test_patch_series_pass(
