@@ -5,6 +5,8 @@ import json
 import pytest
 import tuxsuite
 
+from unittest import mock
+
 
 @pytest.fixture
 def test_json():
@@ -29,12 +31,14 @@ def test_json():
         "result": "pass",
         "results": {"boot": "pass", "ltp-smoke": "pass"},
         "plan": "1t2ghz9i7oeLHa2pW1a8EsR1RLP",
+        "download_url": "https://storage.tuxsuite.com/uid1/",
         "waiting_for": None,
         "boot_args": None,
         "provisioning_time": "2021-05-25T19:58:44.093685",
         "running_time": "2021-05-25T19:58:44.493457",
         "finished_time": "2021-05-25T19:59:45.311189",
         "duration": 61,
+        "test_name": "",
     }
     return json.dumps(test).encode("utf-8")
 
@@ -62,12 +66,14 @@ def test_error_json():
         "result": "error",
         "results": {"boot": "pass", "ltp-smoke": "pass"},
         "plan": "1t2ghz9i7oeLHa2pW1a8EsR1RLP",
+        "download_url": "https://storage.tuxsuite.com/uid1/",
         "waiting_for": None,
         "boot_args": None,
         "provisioning_time": "2021-05-25T19:58:44.093685",
         "running_time": "2021-05-25T19:58:44.493457",
         "finished_time": "2021-05-25T19:59:45.311189",
         "duration": 61,
+        "test_name": "",
     }
     return json.dumps(test).encode("utf-8")
 
@@ -95,12 +101,14 @@ def test_fail_json():
         "result": "fail",
         "results": {"boot": "pass", "ltp-smoke": "pass"},
         "plan": "1t2ghz9i7oeLHa2pW1a8EsR1RLP",
+        "download_url": "https://storage.tuxsuite.com/uid1/",
         "waiting_for": None,
         "boot_args": None,
         "provisioning_time": "2021-05-25T19:58:44.093685",
         "running_time": "2021-05-25T19:58:44.493457",
         "finished_time": "2021-05-25T19:59:45.311189",
         "duration": 61,
+        "test_name": "",
     }
     return json.dumps(test).encode("utf-8")
 
@@ -128,12 +136,14 @@ def test_canceled_json():
         "result": "canceled",
         "results": {"boot": "canceled", "ltp-smoke": "canceled"},
         "plan": "1t2ghz9i7oeLHa2pW1a8EsR1RLP",
+        "download_url": "https://storage.tuxsuite.com/uid1/",
         "waiting_for": None,
         "boot_args": None,
         "provisioning_time": "2021-05-25T19:58:44.093685",
         "running_time": "2021-05-25T19:58:44.493457",
         "finished_time": "2021-05-25T19:59:45.311189",
         "duration": 61,
+        "test_name": "",
     }
     return json.dumps(test).encode("utf-8")
 
@@ -161,12 +171,14 @@ def test_unknown_json():
         "result": "unknown",
         "results": {"boot": "unknown", "ltp-smoke": "unknown"},
         "plan": "1t2ghz9i7oeLHa2pW1a8EsR1RLP",
+        "download_url": "https://storage.tuxsuite.com/uid1/",
         "waiting_for": None,
         "boot_args": None,
         "provisioning_time": "2021-05-25T19:58:44.093685",
         "running_time": "2021-05-25T19:58:44.493457",
         "finished_time": "2021-05-25T19:59:45.311189",
         "duration": 61,
+        "test_name": "",
     }
     return json.dumps(test).encode("utf-8")
 
@@ -197,12 +209,14 @@ def test_list_json():
                 "result": "pass",
                 "results": {"boot": "pass", "ltp-smoke": "pass"},
                 "plan": "1t2gzLqkWHi2ldxDETNMVHPYBYo",
+                "download_url": "https://storage.tuxsuite.com/uid1/",
                 "waiting_for": None,
                 "boot_args": None,
                 "provisioning_time": "2021-05-25T20:01:03.057613",
                 "running_time": "2021-05-25T20:01:03.318610",
                 "finished_time": "2021-05-25T20:01:54.490403",
                 "duration": 51,
+                "test_name": "",
             },
             {
                 "project": "tuxsuite/senthil",
@@ -225,12 +239,14 @@ def test_list_json():
                 "result": "canceled",
                 "results": {"boot": "canceled", "ltp-smoke": "canceled"},
                 "plan": "1t2gzLqkWHi2ldxDETNMVHPYBYo",
+                "download_url": "https://storage.tuxsuite.com/uid2/",
                 "waiting_for": None,
                 "boot_args": None,
                 "provisioning_time": "2021-05-25T20:01:03.057613",
                 "running_time": "2021-05-25T20:01:03.318610",
                 "finished_time": "2021-05-25T20:01:54.490403",
                 "duration": 51,
+                "test_name": "",
             },
             {
                 "project": "tuxsuite/senthil",
@@ -253,12 +269,14 @@ def test_list_json():
                 "result": "unknown",
                 "results": {"boot": "unknown", "ltp-smoke": "unknown"},
                 "plan": "1t2gzLqkWHi2ldxDETNMVHPYBYo",
+                "download_url": "https://storage.tuxsuite.com/uid3/",
                 "waiting_for": None,
                 "boot_args": None,
                 "provisioning_time": "2021-05-25T20:01:03.057613",
                 "running_time": "2021-05-25T20:01:03.318610",
                 "finished_time": "2021-05-25T20:01:54.490403",
                 "duration": 51,
+                "test_name": "",
             },
         ],
         "next": None,
@@ -300,7 +318,7 @@ def result_json():
     return json.dumps(result).encode("utf-8")
 
 
-def test_test_handle_get(mocker, test_json, config, response, monkeypatch):
+def test_test_handle_get(mocker, test_json, config, response, monkeypatch, tmp_path):
     monkeypatch.setattr(
         sys, "argv", ["tuxsuite", "test", "get", "1t2giU7PHbVdarV0ZFIohd0PvFb"]
     )
@@ -310,7 +328,7 @@ def test_test_handle_get(mocker, test_json, config, response, monkeypatch):
     tuxsuite.cli.main()
     assert get_req.call_count == 1
 
-    # Test json out
+    # Test --json
     monkeypatch.setattr(
         sys,
         "argv",
@@ -322,6 +340,27 @@ def test_test_handle_get(mocker, test_json, config, response, monkeypatch):
     tuxsuite.cli.main()
     assert get_req.call_count == 1
 
+    # Test --json-out
+    json_path = f"{tmp_path}/test.json"
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "tuxsuite",
+            "test",
+            "get",
+            "1t2giU7PHbVdarV0ZFIohd0PvFb",
+            "--json-out",
+            json_path,
+        ],
+    )
+    response.status_code = 200
+    response._content = test_json
+    get_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert get_req.call_count == 1
+    assert len(json.load(open(json_path, "r"))) > 0
+
     # Test failure case when the response is not 200
     response.status_code = 500
     get_req = mocker.patch("requests.get", return_value=response)
@@ -329,6 +368,110 @@ def test_test_handle_get(mocker, test_json, config, response, monkeypatch):
         tuxsuite.cli.main()
     assert get_req.call_count == 1
 
+    # case: --list-artifacts
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "tuxsuite",
+            "test",
+            "get",
+            "1t2giU7PHbVdarV0ZFIohd0PvFb",
+            "--list-artifacts",
+        ],
+    )
+    response.status_code = 200
+    response._content = test_json
+    get_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert get_req.call_count == 2
+    assert "storage.tuxsuite.com" in get_req.call_args[0][0]
+
+    # case: --list-artifacts with path
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "tuxsuite",
+            "test",
+            "get",
+            "1t2giU7PHbVdarV0ZFIohd0PvFb",
+            "--list-artifacts",
+            "dir_A",
+        ],
+    )
+    response.status_code = 200
+    response._content = test_json
+    get_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert get_req.call_count == 2
+    assert "storage.tuxsuite.com" in get_req.call_args[0][0]
+    assert "/dir_A/" in get_req.call_args[0][0]
+
+    # case: --list-artifacts with download option
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "tuxsuite",
+            "test",
+            "get",
+            "1t2giU7PHbVdarV0ZFIohd0PvFb",
+            "--list-artifacts",
+            "-d",
+            str(tmp_path),
+        ],
+    )
+    response.status_code = 200
+    response._content = test_json
+    get_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert get_req.call_count == 2
+    assert "storage.tuxsuite.com" in get_req.call_args[0][0]
+
+    # case: --list-artifacts with path and  download option
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "tuxsuite",
+            "test",
+            "get",
+            "1t2giU7PHbVdarV0ZFIohd0PvFb",
+            "--list-artifacts",
+            "dir_A",
+            "-d",
+            str(tmp_path),
+        ],
+    )
+    response.status_code = 200
+    response._content = test_json
+    get_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert get_req.call_count == 2
+    assert "storage.tuxsuite.com" in get_req.call_args[0][0]
+    assert "/dir_A" in get_req.call_args[0][0]
+
+    # case: --download-artifacts with path
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "tuxsuite",
+            "test",
+            "get",
+            "1t2giU7PHbVdarV0ZFIohd0PvFb",
+            "--download-artifacts",
+            str(tmp_path),
+        ],
+    )
+    response.status_code = 200
+    response._content = test_json
+    get_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert get_req.call_count == 2
+    assert "storage.tuxsuite.com" in get_req.call_args[0][0]
+
 
 def test_test_handle_cancel(mocker, test_json, config, response, monkeypatch):
     monkeypatch.setattr(
@@ -404,18 +547,21 @@ def test_test_handle_get_fail(mocker, test_fail_json, config, response, monkeypa
     assert get_req.call_count == 1
 
 
-def test_test_handle_list(mocker, test_list_json, config, response, monkeypatch):
-    monkeypatch.setattr(sys, "argv", ["tuxsuite", "test", "list"])
+def test_test_handle_list(
+    mocker, test_list_json, config, response, monkeypatch, tmp_path
+):
+    monkeypatch.setattr(sys, "argv", ["tuxsuite", "test", "list", "--limit", "1"])
     monkeypatch.setattr("sys.stdout.isatty", lambda: True)
+    mocker.patch("builtins.input", side_effect=KeyboardInterrupt)
     response.status_code = 200
     response._content = test_list_json
     list_req = mocker.patch("requests.get", return_value=response)
     with pytest.raises(SystemExit) as exit:
         tuxsuite.cli.main()
     assert exit.value.code == 0
-    assert list_req.call_count == 2
+    assert list_req.call_count == 1
 
-    # Test json out
+    # Test --json
     monkeypatch.setattr(sys, "argv", ["tuxsuite", "test", "list", "--json"])
     response.status_code = 200
     response._content = test_list_json
@@ -424,6 +570,19 @@ def test_test_handle_list(mocker, test_list_json, config, response, monkeypatch)
     assert exit.value.code == 0
     assert list_req.call_count == 1
 
+    # Test --json-out
+    json_path = f"{tmp_path}/test.json"
+    monkeypatch.setattr(
+        sys, "argv", ["tuxsuite", "test", "list", "--json-out", json_path]
+    )
+    response.status_code = 200
+    response._content = test_list_json
+    list_req = mocker.patch("requests.get", return_value=response)
+    tuxsuite.cli.main()
+    assert exit.value.code == 0
+    assert list_req.call_count == 1
+    assert len(json.load(open(json_path, "r"))) > 0
+
     # Test failure case when the response is not 200
     response.status_code = 500
     list_req = mocker.patch("requests.get", return_value=response)
@@ -436,27 +595,53 @@ def test_test_handle_logs(mocker, config, response, monkeypatch):
     monkeypatch.setattr(
         sys, "argv", ["tuxsuite", "test", "logs", "1yiYkYq26HbT5i304xElM5Czj2d"]
     )
-    response.status_code = 200
-    response._content = b"""- {"dt": "2021-09-27T09:58:08.499180", "lvl": "info", "msg": "msg-1"}
+
+    def mock_request(url, **kwargs):
+        if (
+            url
+            == "https://tuxapi.tuxsuite.com/v1/groups/tuxsuite/projects/tux/tests/1yiYkYq26HbT5i304xElM5Czj2d"
+        ):
+            return mock.Mock(
+                status_code=200,
+                json=lambda: {
+                    "download_url": (
+                        "https://storage.tuxsuite.com/public/tuxsuite/tux/tests/"
+                        "1yiYkYq26HbT5i304xElM5Czj2d/"
+                    )
+                },
+            )
+        elif (
+            url
+            == "https://storage.tuxsuite.com/public/tuxsuite/tux/tests/1yiYkYq26HbT5i304xElM5Czj2d/lava-logs.yaml"
+        ):
+            return mock.Mock(
+                status_code=200,
+                text=b"""- {"dt": "2021-09-27T09:58:08.499180", "lvl": "info", "msg": "msg-1"}
 - {"dt": "2021-09-27T09:58:08.499454", "lvl": "info", "msg": "msg-2"}
 - {"dt": "2021-09-27T09:58:08.500845", "lvl": "debug", "msg": "msg-3"}
-"""
-    logs_req = mocker.patch("requests.get", return_value=response)
-    tuxsuite.cli.main()
-    assert logs_req.call_count == 1
+""",
+            )
+
+    with mock.patch("requests.get", side_effect=mock_request) as logs_req:
+        tuxsuite.cli.main()
+        assert logs_req.call_count == 2
 
     # test raw output
-    mocker.resetall()
     monkeypatch.setattr(
         sys,
         "argv",
         ["tuxsuite", "test", "logs", "1yiYkYq26HbT5i304xElM5Czj2d", "--raw"],
     )
-    response.status_code = 200
-    response._content = b""
-    tuxsuite.cli.main()
+    with mock.patch("requests.get", side_effect=mock_request) as logs_req:
+        tuxsuite.cli.main()
+        assert logs_req.call_count == 2
+
+    # Test failure case when the response is not 200
+    response.status_code = 500
     logs_req = mocker.patch("requests.get", return_value=response)
-    tuxsuite.cli.main()
+    with pytest.raises(NotImplementedError):
+        tuxsuite.cli.main()
+    assert logs_req.call_count == 1
     assert logs_req.call_count == 1
 
     # Test failure case when the response is not 200
@@ -471,11 +656,30 @@ def test_test_handle_results(mocker, config, result_json, response, monkeypatch)
     monkeypatch.setattr(
         sys, "argv", ["tuxsuite", "test", "results", "1yiYkYq26HbT5i304xElM5Czj2d"]
     )
-    response.status_code = 200
-    response._content = result_json
-    logs_req = mocker.patch("requests.get", return_value=response)
-    tuxsuite.cli.main()
-    assert logs_req.call_count == 1
+
+    def mock_request(url, **kwargs):
+        if (
+            url
+            == "https://tuxapi.tuxsuite.com/v1/groups/tuxsuite/projects/tux/tests/1yiYkYq26HbT5i304xElM5Czj2d"
+        ):
+            return mock.Mock(
+                status_code=200,
+                json=lambda: {
+                    "download_url": (
+                        "https://storage.tuxsuite.com/public/tuxsuite/tux/tests/"
+                        "1yiYkYq26HbT5i304xElM5Czj2d/"
+                    )
+                },
+            )
+        elif (
+            url
+            == "https://storage.tuxsuite.com/public/tuxsuite/tux/tests/1yiYkYq26HbT5i304xElM5Czj2d/results.json"
+        ):
+            return mock.Mock(status_code=200, text=result_json)
+
+    with mock.patch("requests.get", side_effect=mock_request) as logs_req:
+        tuxsuite.cli.main()
+        assert logs_req.call_count == 2
 
     # test raw output
     mocker.resetall()
@@ -484,11 +688,9 @@ def test_test_handle_results(mocker, config, result_json, response, monkeypatch)
         "argv",
         ["tuxsuite", "test", "results", "1yiYkYq26HbT5i304xElM5Czj2d", "--raw"],
     )
-    response._content = b""
-    tuxsuite.cli.main()
-    results_req = mocker.patch("requests.get", return_value=response)
-    tuxsuite.cli.main()
-    assert results_req.call_count == 1
+    with mock.patch("requests.get", side_effect=mock_request) as logs_req:
+        tuxsuite.cli.main()
+        assert logs_req.call_count == 2
 
     # Test failure case when the response is not 200
     response.status_code = 500
