@@ -39,6 +39,5 @@ def test_download(build, get, response, mocker, tmp_path):
 def test_download_file(get, response, tmp_path):
     dest = tmp_path / "foo.txt"
     response.raw = BytesIO(b"Hello World\n")
-    with dest.open("wb") as f:
-        download_file("https://builds.test/12312321/foo.txt", f)
+    download_file("https://builds.test/12312321/foo.txt", dest)
     assert dest.read_text() == "Hello World\n"
