@@ -44,6 +44,10 @@ from tuxsuite.cli.keys import (
     handlers as keys_handlers,
     setup_parser as keys_parser,
 )
+from tuxsuite.cli.trigger import (
+    handlers as trigger_handlers,
+    setup_parser as trigger_parser,
+)
 
 
 def setup_parser():
@@ -68,7 +72,7 @@ def setup_parser():
 
     bake = sub_parser.add_parser(
         "bake",
-        help="Do an OE/Yocto build with bitbake like 'tuxsuite bake submit <build-definition.json>'",
+        help="Do an OE/Yocto build with bitbake like 'tuxsuite bake submit <build-definition.yaml>'",
     ).add_subparsers(dest="sub_command", help="Commands")
     bake.required = True
     bake_cmds = bake_parser(bake)
@@ -115,6 +119,12 @@ def setup_parser():
     keys.required = True
     keys_cmds = keys_parser(keys)
 
+    trigger = sub_parser.add_parser("trigger", help="Trigger.").add_subparsers(
+        dest="sub_command", help="Commands"
+    )
+    trigger.required = True
+    trigger_cmds = trigger_parser(trigger)
+
     return (
         parser,
         {
@@ -126,6 +136,7 @@ def setup_parser():
             "project": project_cmds,
             "test": test_cmds,
             "results": results_cmds,
+            "trigger": trigger_cmds,
         },
     )
 
@@ -156,6 +167,14 @@ def main():
 
     (options, extra_arguments) = parser.parse_known_args()
 
+    # SPECIAL CASE: "tuxsuite plan create|execute" sub-sub-command does not require
+    # the config in order to generate / create a plan file / run a plan locally
+    if options.command == "plan" and options.sub_command == "create":
+        return plan_handlers[options.sub_command](options, extra_arguments)
+
+    if options.command == "plan" and options.sub_command == "execute":
+        return plan_handlers[options.sub_command](options, extra_arguments)
+
     cfg = load_config()
     # Handle command
     if options.command == "results":
@@ -174,6 +193,8 @@ def main():
         return project_handlers[options.sub_command](options, extra_arguments, cfg)
     elif options.command == "keys":
         return keys_handlers[options.sub_command](options, extra_arguments, cfg)
+    elif options.command == "trigger":
+        return trigger_handlers[options.sub_command](options, extra_arguments, cfg)
     else:
         error("Unknown sub command")
         sys.exit(2)
