@@ -147,15 +147,19 @@ class Test(Base):
     kernel: str
     modules: str
     tests: List[str]
+    test_name: str
     state: str
     result: str
     results: Dict[str, str]
     plan: str
     waiting_for: str
+    download_url: str
     provisioning_time: str
     running_time: str
     finished_time: str
     extra: Dict
+    overlays: List = field(default_factory=list)
+    waited_by: List = field(default_factory=list)
     duration: int = None
     boot_args: str = None
     user: str = None
@@ -206,6 +210,10 @@ class Bitbake(Base):
     def __lt__(self, other):
         pass
 
+    def url(self):
+        (grp, prj) = self.project.split("/")
+        return f"/v1/groups/{grp}/projects/{prj}/oebuilds/{self.uid}"
+
     def get_builds_message(self, icon, color, msg):
         return (
             f"{self.uid} {icon} {color}{msg}{colors.reset} with container {self.container} "
@@ -242,8 +250,20 @@ class Plan(Base):
     def as_json(self):
         return json.dumps(self.as_dict())
 
-    def _tests_wait_for(self, uid):
-        return [t for t in self.tests if t.waiting_for == uid]
+    def _dependent_tests(self, uid):
+        return [
+            t
+            for t in self.tests
+            if t.waiting_for and t.waiting_for.split("#")[1] == uid
+        ]
+
+    def _tests_wait_for(self, build_uid, get_sanity=False):
+        sanity_test = None
+        tests = self._dependent_tests(build_uid)
+        if len(tests) == 1 and tests[0].waited_by:
+            sanity_test = tests[0]
+            tests = self._dependent_tests(sanity_test.uid)
+        return (tests, sanity_test) if get_sanity else tests
 
     def filter_builds(self, f):
         return sorted([b for b in self.all_builds if f(self, b)])
