@@ -1,33 +1,35 @@
 # -*- coding: utf-8 -*-
 
+import argparse
+import json
 import os
 import sys
+import re
+import tempfile
+from pathlib import Path
+
 import tuxsuite.download
 import tuxsuite.cli.colors as colors
+from tuxsuite import schema
+from tuxsuite.cli.yaml import yaml_load, YamlError
 
 from itertools import chain
+from tuxsuite.schema import plan as plan_schema, SchemaError
+from tuxsuite.requests import get as GET
 from tuxsuite.cli.requests import get
 from urllib.parse import urlparse
-from tuxsuite.utils import (
-    ResultState,
-    result_states,
-)
+from tuxsuite.utils import error, ResultState, result_states
 
 
 LIMIT = 50
 
 
-def error(msg):
-    sys.stderr.write(f"Error: {msg}\n")
-    sys.exit(1)
-
-
 def datediff(one, two):
     if one is None:
         return two
 
     if one == two:
-        return f"{colors.white}{two}{colors.reset}"
+        return f"{colors.white}{two}{colors.reset}" if sys.stdout.isatty() else two
 
     index = 0
     for o, t in zip(one, two):
@@ -35,25 +37,27 @@ def datediff(one, two):
             break
         index += 1
 
-    return f"{colors.white}{two[0:index]}{colors.reset}{two[index:]}"
+    return (
+        f"{colors.white}{two[0:index]}{colors.reset}{two[index:]}"
+        if sys.stdout.isatty()
+        else f"{two[0:index]}{two[index:]}"
+    )
 
 
-def fetch_next(cmd_cls, cfg, url, next_token, limit):
+def fetch_next(cmd_cls, cfg, url, next_token):
+    # Return next 50 items
+    if not next_token:
+        return ([], None)
     ret = get(cfg, url, params={"start": next_token})
-    cmd_name = url.split("/")[-1]
     next_token = ret.json()["next"]
-    res = [cmd_cls.new(**r) for r in ret.json()["results"][:limit]]
-    if next_token:
-        input(f"\nPress [ENTER] to see next list of {cmd_name}, or Ctrl-C to quit:\n")
-    else:
-        sys.exit(0)
+    res = [cmd_cls.new(**r) for r in ret.json()["results"]]
     return (res, next_token)
 
 
 def key_value(s):
-    if s.count("=") != 1:
+    if s.count("=") == 0:
         error(f"Key Value pair not valid: {s}")
-    parts = s.split("=")
+    parts = s.split("=", 1)
     return (parts[0], "=".join(parts[1:]))
 
 
@@ -211,11 +215,15 @@ def show_log(build, download, output_dir):
         # Authorization header in case of private build
         headers = {"Authorization": build.headers["Authorization"]}
         if build.status.get("download_url"):
-            tuxsuite.download.download_file(
-                os.path.join(build.status["download_url"], "build.log"),
-                sys.stderr.buffer,
-                headers=headers,
-            )
+            with tempfile.NamedTemporaryFile() as build_log:
+                tuxsuite.download.download_file(
+                    os.path.join(build.status["download_url"], "build.log"),
+                    Path(build_log.name),
+                    headers=headers,
+                )
+                build_log.seek(0)
+                for line in build_log:
+                    sys.stderr.buffer.write(line)
         else:
             error("download_url empty")
 
@@ -340,3 +348,107 @@ def get_remote_version(config):
         error("Unable to fetch remote versions")
     else:
         return ret.json()
+
+
+class overlay_action(argparse.Action):
+    def __call__(self, parser, namespace, values, option_string=None):
+        entries = len(values)
+
+        pairs = getattr(namespace, self.dest, [])
+
+        if entries > 2:
+            parser.error(
+                f"More than 2 arguments passed for {self.dest} options. Please check help options"
+            )
+
+        if entries == 1:
+            pairs.append([values[0], "/"])
+        else:
+            pairs.append([values[0], values[1]])
+        setattr(namespace, self.dest, pairs)
+
+
+def load_plan(plan_file):
+    try:
+        if urlparse(str(plan_file)).scheme in ["http", "https"]:
+            response = GET(plan_file)
+            if response.status_code != 200:
+                error(f"Failed to get plan file: {response.text}")
+            contents = response.text
+        else:
+            contents = open(plan_file).read()
+        plan_yaml = yaml_load(contents, True)
+        plan_schema()(plan_yaml)
+        return plan_yaml
+    except (FileNotFoundError, YamlError, SchemaError) as e:
+        error(str(e))
+
+
+def is_json(json_str):
+    try:
+        json.loads(json_str)
+        return True
+    except (json.JSONDecodeError, TypeError):
+        return False
+
+
+def load_build_definition(definition_file):
+    def deprecated_warning():
+        print(
+            "⚠️ WARNING: Support for build definition files in JSON format is deprecated and will be removed in release v1.44.0 or not later than a release on 6th August, 2025. Please migrate to the YAML format."
+        )
+
+    try:
+        contents = None
+        if urlparse(str(definition_file)).scheme in ["http", "https"]:
+            response = GET(definition_file)
+            response.raise_for_status()
+            contents = response.text
+            if is_json(contents):
+                deprecated_warning()
+        else:
+            if is_json(Path(definition_file).read_text()):
+                deprecated_warning()
+            contents = open(definition_file).read()
+        contents = yaml_load(contents, safe=True)
+        # Validate schema
+        schema.tuxbake_schema()(contents)
+        return contents
+    except YamlError as ex:
+        error(f"E: Invalid build definition file\n{ex}")
+    except (FileNotFoundError, Exception) as e:
+        error(str(e))
+
+
+def get_cb_headers(headers):
+    headers_dict = {}
+    for header in headers:
+        if ":" not in header:
+            error(
+                f"{header} is not a valid header. "
+                "Require a key value pair separated by a ':' (colon)."
+            )
+        key, value = header.split(":")
+        headers_dict.update({key: value.lstrip()})
+    return headers_dict
+
+
+def is_header(header):
+    """Validate if header is valid"""
+    if ":" not in header:
+        error(
+            f"{header} is not a valid header. "
+            "Require a key value pair separated by a ':' (colon)."
+        )
+
+
+def is_valid_email(email):
+    if re.match(r"[^@]+@[^@]+\.[^@]+", email):
+        return True
+    error(f"{email} is not a valid email-id.")
+
+
+def validate_notify_emails(notify_emails):
+    if notify_emails:
+        for email in notify_emails:
+            is_valid_email(email)
